Package: compcodeR
Type: Package
Title: RNAseq data simulation, differential expression analysis and
        performance comparison of differential expression methods
Version: 1.49.0
Authors@R: c(
    person("Charlotte", "Soneson", role = c("aut", "cre"), 
           email = "charlottesoneson@gmail.com", comment = c(ORCID = "0000-0003-3833-2169")),
    person("Paul", "Bastide", role = "aut",
           email = "paul.bastide@umontpellier.fr", comment = c(ORCID = "0000-0002-8084-9893")),
    person("Mélina", "Gallopin", role = "aut",
           email = "melina.gallopin@u-psud.fr", comment = c(ORCID = "0000-0002-2431-7825 "))
           )
Description: This package provides extensive functionality for
    comparing results obtained by different methods for
    differential expression analysis of RNAseq data. It also
    contains functions for simulating count data. Finally, it provides 
    convenient interfaces to several packages for performing the 
    differential expression analysis. These can also be used as templates
    for setting up and running a user-defined differential analysis 
    workflow within the framework of the package.
Depends: R (>= 4.0), sm
Imports: knitr (>= 1.2), markdown, ROCR, lattice (>= 0.16), gplots,
        gtools, caTools, grid, KernSmooth, MASS, ggplot2, stringr,
        modeest, edgeR, limma, vioplot, methods, stats, utils, ape,
        phylolm, matrixStats, grDevices, graphics, rmarkdown, shiny,
        shinydashboard
Suggests: BiocStyle, EBSeq, DESeq2 (>= 1.1.31), genefilter, NOISeq,
        TCC, NBPSeq (>= 0.3.0), phytools, phangorn, testthat, ggtree,
        tidytree, statmod, covr, sva, tcltk
Enhances: rpanel, DSS
License: GPL (>= 2)
VignetteBuilder: knitr
biocViews: ImmunoOncology, RNASeq, DifferentialExpression
RoxygenNote: 7.2.3
URL: https://github.com/csoneson/compcodeR
BugReports: https://github.com/csoneson/compcodeR/issues
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/compcodeR
git_branch: devel
git_last_commit: 734da12
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:57:51 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Paul Bastide [aut] (ORCID: <https://orcid.org/0000-0002-8084-9893>),
  Mélina Gallopin [aut] (ORCID: <https://orcid.org/0000-0002-2431-7825>)
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>
