#' RNAseq data simulation, differential expression analysis and
#' performance comparison of differential expression methods
#'
#' This package provides extensive functionality for comparing
#' results obtained by different methods for differential expression analysis
#' of RNAseq data. It also contains functions for simulating count data and
#' interfaces to several packages for performing the differential expression analysis.
#' @name compcodeR-package
#' @aliases compcodeR
#' @docType package
#' @import knitr markdown ROCR lattice gplots gtools caTools grid KernSmooth MASS ggplot2 stringr modeest limma edgeR vioplot methods rmarkdown sm
#' @title RNAseq data simulation, differential expression analysis and
#' performance comparison of differential expression methods
#' @author Charlotte Soneson
#' @keywords package
NULL
