% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulatePhyloPoissonLogNormal.R
\name{scale_variance_process}
\alias{scale_variance_process}
\title{Scale the variances}
\usage{
scale_variance_process(
  log_variance_phylo,
  tree,
  model.process,
  selection.strength
)
}
\arguments{
\item{tree}{a dated phylogenetic tree of class \code{\link[ape]{phylo}} with `samples.per.cond * 2` species.}

\item{model.process}{the process to be used for phylogenetic simulations. One of "BM" or "OU", default to "BM".}

\item{selection.strength}{if the process is "OU", the selection strength parameter.}
}
\value{
A matrix N * P of factors to multiply the simulated phylogenetic residuals.
}
\description{
Scale the variances of the process simulation so that they are equal to log_variance_phylo.
}
\keyword{internal}
